
/*==================================================================
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *===============================================================*/

package contactlist.struts.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import contactlist.struts.utilities.AIMException;
import contactlist.struts.utilities.AIMWrapperException;


/**
 * DB Broker generic function
 * 
 * Creation date: (11/30/2003 10:00:00 PM)
 * @author: Tim Hanis
 */
public abstract class AbstractDBBroker extends ContactListBroker {

	// Cache for datasource
	protected static final HashMap datasources = new HashMap(3);	
	protected String datasourceName = "";
	
	protected static final int dbTimeout = 120; // seconds
	protected final String SUBCONTEXT = "jdbc/";
	protected final String INITIAL_CONTEXT = "com.ibm.websphere.naming.WsnInitialContextFactory";	


	/**
	 * Close the database resultSet, statement and connection.
	 * @param connection java.sql.Connection
	 * @param statement java.sql.Statement
	 * @param resultSet java.sql.ResultSet
	 */
	protected final void close(Connection connection, Statement statement, ResultSet resultSet)
			throws AIMException {
				
		try {
			if (connection != null && !connection.isClosed())
				connection.close();
			if (statement != null)
				statement.close();
			if (resultSet != null)
				resultSet.close();
		} catch (SQLException ignored) {}

	}
	
	
	/**
	 * Execute the statement and return the result set 
	 * @param statement java.sql.PreparedStatement
	 * @return resultSet java.sql.ResultSet
	 */
	protected final ResultSet execute(PreparedStatement statement)
		throws AIMException {

		try {
			statement.setQueryTimeout(dbTimeout);
			boolean resultSetAvailable = statement.execute();
			if (resultSetAvailable)
				return statement.getResultSet();
			return null;
		
		} catch (SQLException e) {
			throw new AIMWrapperException("SQL exception while executing sql query", e);	
		} catch (Exception e) {
			throw new AIMWrapperException("Non-SQL exception while executing sql query", e);
		}

	}
	
	
	/**
	 * Return a connection from the datasource pool.
	 * @return java.sql.Connection
	 */


	protected final Connection getConnection() throws AIMException {
		try {
			return getDataSource().getConnection();
		} catch (SQLException e) {
			throw new AIMWrapperException("SQLException while getting a database connection.  Check parameter settings.",
				e);
		}
	}
	
	
	/**
	 * Return the datasource to use for pooled connections.
	 * @return javax.sql.DataSource
	 */
	private final DataSource getDataSource() throws AIMException {
		
		
		// Return the datasource from the cache if available
		DataSource dataSource = (DataSource) datasources.get(datasourceName);
		if (dataSource != null)
			return dataSource;
		
		// Retrieve a DataSource through the JNDI Naming Service
		Properties parms = new Properties();
		parms.setProperty(Context.INITIAL_CONTEXT_FACTORY, INITIAL_CONTEXT);

		// Create the Initial Naming Context
		Context context = null;
		try {
			context = new InitialContext(parms);
		} catch (NamingException e) {
			throw new AIMWrapperException("Naming exception while creating JNDI context", e);
		}

		// Lookup through the naming service to retrieve a DataSource object
		try {
			dataSource = (DataSource)context.lookup(SUBCONTEXT + datasourceName);
		} catch (NamingException e) {
			throw new AIMWrapperException("Naming exception while performing datasource lookup.", e);
		}

		// Put it in the cache and return it
		datasources.put(datasourceName, dataSource);
		return dataSource;
	
	}
	
	
}