/*==================================================================
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *===============================================================*/
package contactlist.struts.persistence;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import contactlist.forms.Contact;
import contactlist.struts.utilities.AIMException;

/**
 * Memory broker implementation 
 * 
 * Creation date: (11/30/2003 10:00:00 PM)
 * @author: Tim Hanis
 */
public class ContactListMemBroker extends ContactListBroker {

	protected List contactList = new ArrayList();
	protected Log log = null;

	/**
	 *  Null Constructor not visible
	 */
	private ContactListMemBroker() {
		super();
	}

	/**
	 *  Broker initializer
	 */
	public static void initialize() {
		ContactListMemBroker broker = new ContactListMemBroker();
		broker.log = LogFactory.getLog("contactlist.struts.persistence");
		brokerInstance = broker;
	}

	public void populate() {
		Contact contact = null;
		
		for (int i = 0; i < 5; i++) {
			contact=new Contact();
			
			contact.setOid("" + i);
			contact.setOwner("wpsadmin");
			contact.setFirstName("First Name" + i);
			contact.setLastName("Last Name" + i);
			contact.setEmail("email" + i);
			contact.setTitle("Title" + i);
			contact.setCompany("Company" + i);
			contact.setBusinessPhone("Business Phone" + i);
			contact.setMobilePhone("Mobile Phone" + i);
			contact.setFaxPhone("Fax Phone" + i);
			contact.setWeb("WebSite" + i);
			contact.setAddress("Address" + i);
			contact.setCity("city" + i);
			contact.setState("state" + i);
			contact.setZip("zip" + i);
			contact.setCountry("USA" + i);

			try {
				saveContact(contact);
				//System.out.println(contact);
			} catch (Exception e) {
				System.out.println(e);
			}
		}
	}

	/**
	 * Get the Contact information given an Contact number
	 * @param owner java.lang.String
	 * @param contactOID java.lang.String
	 * @return Contact
	 */
	public Contact getContact(String owner, String oid) throws AIMException {

		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Getting contact " + oid + " for user " + owner);
		Contact contact = null;
		for (int i = 0; i < contactList.size(); i++) {
			contact = (Contact) contactList.get(i);

			if (owner.equals(contact.getOwner())
				&& oid.equals(contact.getOid())) {
				return contact;
			}
		}

		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Got contact " + contact);

		return contact;
	}

	/**
	 * Create a Contact
	 * @param Contact
	 */
	public void saveContact(Contact contact) throws AIMException {

		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Saving contact for user " + contact.getOwner());

		contactList.add(contact);

	}

	/**
	 * Delete a Contact
	 * @param Contact
	 */
	public void deleteContact(String owner, String oid) throws AIMException {

		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Deleting contact " + oid + " for user " + owner);

		Contact contact = null;
		for (int i = 0; i < contactList.size(); i++) {
			contact = (Contact) contactList.get(i);
			if (owner.equals(contact.getOwner())
				&& oid.equals(contact.getOid())) {
				contactList.remove(i);
				return;
			}
		}
	}

	/**
	 * Get the contact list. 
	 * @param filter java.lang.String
	 * @return ContactList
	 */
	public List getContactList(String owner) throws AIMException {

		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Getting contact list for user " + owner);

		List contactListForUser = new ArrayList();
		Contact contact = null;
		for (int i = 0; i < contactList.size(); i++) {
			contact = (Contact) contactList.get(i);
			if (owner.equals(contact.getOwner())) {
				contactListForUser.add(contact);
			}
		}
		return contactListForUser;

	}

}
