//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.web;

import java.io.IOException;
import java.io.Serializable;
import java.util.PropertyResourceBundle;
import java.text.MessageFormat;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.samples.bank.ejb.AccountKey;
import com.ibm.websphere.samples.bank.ejb.AccountLocal;
import com.ibm.websphere.samples.bank.ejb.AccountLocalHome;

import java.util.logging.*;

/**
 * This servlet creates account for a particular customer using the customer 
 * id. It creates a savings or the checkings account.
 */

public class CreateAccountServlet extends HttpServlet implements Serializable {

    // EJB access.
    private AccountLocalHome accountHome = null;
    private static final String PROPFILE = "/WEB-INF/properties/bank_messages";
    PropertyResourceBundle bundle = (PropertyResourceBundle) PropertyResourceBundle.getBundle(PROPFILE);
    private final static String ACCOUNT_JNDI_NAME = "java:comp/env/ejb/Account";

    static final String className = CreateAccountServlet.class.getName();
    private Logger messageLogger;
    private static final String rbName = null; // No localization is used for logging.

    /**
    * Initialize the servlet
    */
    public void init(ServletConfig config) throws ServletException {
        final String methodName = "init";
        try {
            messageLogger = Logger.getLogger(className, rbName);
        } catch (Exception e) {
            // Problem initializing logging.  Continue anyway.
        }

        super.init(config);

        try {
            javax.naming.Context ctx = new javax.naming.InitialContext();
            accountHome = (AccountLocalHome) ctx.lookup(ACCOUNT_JNDI_NAME);
        } catch (NamingException e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errNamingInit") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInitExpection") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }
    }
    /**
    * doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);
    }

    /**
    * doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);

    }

    /**
     * @param req
     * @param resp
     * @throws ServletException
     */
    public void performTask(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
        final String methodName = "performTask";

        try {
            PageBean pgBean = new PageBean();
            java.util.ArrayList errmsg = new java.util.ArrayList();
            java.util.ArrayList msg = new java.util.ArrayList();

            long longCustNum = 0;
            int intAcctNum = 0;
            int intAcctType = 0;
            float floatStartBalance = 0;

            javax.servlet.http.HttpSession session = req.getSession(true);
            HistoryBean history = (HistoryBean) session.getAttribute("HistoryBean"); 
            if (history == null) {
      	        history = new HistoryBean(); // creates the bean if there isn't one on the session
            }
            String strCustNum = req.getParameter("customerNum");
            String strAcctNum = req.getParameter("accountNum");
            String strAcctType = req.getParameter("accountType"); // 1 for Savings and 2 for Checking account
            String strStartBalance = req.getParameter("startBalance");

            if (strCustNum == null || strCustNum.trim().length() == 0) {
                errmsg.add(bundle.getString("errCustReq"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errCustReq"));
            } else {
                try {
                    pgBean.setCustomerNum(Long.parseLong(strCustNum));
                    longCustNum = Long.parseLong(strCustNum);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errCustNumValid"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errCustNumValid"));
                }
            }
            if (strAcctNum == null || strAcctNum.trim().length() == 0) {
                errmsg.add(bundle.getString("errAcctNumReq"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errAcctNumReq"));
            } else {
                try {
                    pgBean.setAccountNum(Integer.parseInt(strAcctNum));
                    intAcctNum = Integer.parseInt(strAcctNum);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errAcctNumValid"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errAcctNumValid"));
                }
            }
            if (strAcctType == null || strAcctType.trim().length() == 0) {
                errmsg.add(bundle.getString("errAcctTypeReq"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errAcctTypeReq"));
            } else {
                try {
                    pgBean.setAccountType(Integer.parseInt(strAcctType));
                    intAcctType = Integer.parseInt(strAcctType);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errAcctTypeValid"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errAcctTypeValid"));
                }
            }
            if (strStartBalance == null || strStartBalance.trim().length() == 0) {
                errmsg.add(bundle.getString("errStartBalReq"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errStartBalReq"));
            } else {
                try {
                    pgBean.setBalance(Float.parseFloat(strStartBalance));
                    floatStartBalance = Float.parseFloat(strStartBalance);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errStartBalValid"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errStartBalValid"));
                }
            }

            // Datatypes used to create new account.
            AccountKey key;

            //After page validation if there are no errors 
            if (errmsg.size() == 0) {
                try {
                    //create the account
                    AccountLocal account = accountHome.create(intAcctNum, intAcctType, floatStartBalance, longCustNum);
                    
                    Object[] msgArgs = new Object[1];
                    msgArgs[0] = strAcctNum;
                    MessageFormat msgFormat = new MessageFormat(bundle.getString("succNewAccount"));
                    String msgNewAcct = msgFormat.format(msgArgs);
                    msg.add(msgNewAcct);
                    messageLogger.logp(Level.INFO, className, methodName, msgNewAcct);

                    history.addMessage(msgNewAcct); // adds a message to the command history
                } catch (DuplicateKeyException e) {
                    errmsg.add(bundle.getString("errDuplicateAcct"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errDuplicateAcct") + e.getMessage());
                } catch (CreateException e) {
                    errmsg.add(bundle.getString("errCreateAcct"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errCreateAcct") + e.getMessage());
                    e.printStackTrace();
                } catch (Exception e) {
                    errmsg.add(bundle.getString("errAcctexception"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errAcctexception") + e.getMessage());
                    e.printStackTrace();
                }
            } //end of if

            pgBean.setMessageLines(msg);
            pgBean.setErrormessage(errmsg);

            req.setAttribute("PageBean", pgBean);
            session.setAttribute("HistoryBean", history);// stores it back into the session
            getServletContext().getRequestDispatcher("/jsp/createaccount.jsp").forward(req, resp);
        } catch (Throwable ex) {
            ex.printStackTrace();
            throw new ServletException(ex);
        }
    }
}
